// title      : OpenJSCAD.org Logo
// author     : Leo Blondel, updated by Seth Donoughe
// license    : MIT License
// revision   : 0.004
// tags       : Logo,Intersection,Sphere,Cube
// file       : logo.jscad

function getParameterDefinitions () {
  return [
    {name: 'diameter', caption: 'Diameter of the insert (mm):', type: 'float', initial: 35},
    {name: 'notchsize', caption: 'Size of the notch (mm):', type: 'float', initial: 6},
    {name: 'thickness', caption: 'Thickness of the insert (mm):', type: 'float', initial: 6},
    {name: 'smoothing', caption: 'Smoothing factor:', type: 'float', initial: 10},
    {
      name: 'insertType',
      type: 'choice',
      caption: 'Shape of each well',
      values: ["Rectangular", "Circular"],
      captions: ['Rectangular', 'Circular'],
      initial: 1
    },
    {name: 'embryoDepth', caption: 'Depth of each well (mm):', type: 'float', initial: 0.5, step: 0.01},
    {name: 'embryoRadius', caption: '(If Circular) Diameter of each well (mm):', type: 'float', initial: 0.5, step: 0.01},
    {name: 'embryoLengthX', caption: '(If Rectangular) Length of each well along x-axis (mm):', type: 'float', initial: 0.1, step: 0.01},
    {name: 'embryoLengthY', caption: '(If Rectangular) Length of each well along y-axis (mm):', type: 'float', initial: 0.5, step: 0.01},
    {
      name: 'alleys_bool',
      type: 'choice',
      caption: 'Omit some rows and/or columns of wells to form subgrids?',
      values: ["alleys_yes", "alleys_no"],
      captions: ['Yes', 'No'],
      initial: 1
    },
    {
      name: 'gridtype',
      type: 'choice',
      caption: 'Omit a well in one corner of each complete subgrid?',
      values: ["corner_omitted", "corner_present"],
      captions: ['Yes', 'No'],
      initial: 1
    },
    {name: 'row', caption: 'Number of wells along x-axis:', type: 'float', initial: 10},
    {name: 'column', caption: 'Number of wells along y-axis:', type: 'float', initial: 10},
    {name: 'spacing_x', caption: 'Spacing between wells along x-axis:', type: 'float', initial: 0.5, step: 0.01},
    {name: 'spacing_y', caption: 'Spacing between wells along y-axis:', type: 'float', initial: 0.5, step: 0.01},
    {name: 'skiprow', caption: '(If Skipped) skip every X row:', type: 'int', initial: 5},
    {name: 'skipcol', caption: '(If Skipped) skip every Y row:', type: 'int', initial: 5},
  ];
}

function getadapter()
{
union(
// objects: 1
// object #undefined: triangles: 60
    p = polyhedron({ points: [
        [-12,6,3],
        [-12,-6,3],
        [-12,-6,0],
        [-12,-6,0],
        [-12,6,0],
        [-12,6,3],
        [12,-6,3],
        [12,6,3],
        [12,6,0],
        [12,6,0],
        [12,-6,0],
        [12,-6,3],
        [12,-6,-3],
        [12,6,-3],
        [3.999999761581421,6,-3],
        [3.999999761581421,6,-3],
        [3.999999761581421,-6,-3],
        [12,-6,-3],
        [5.599999904632568,-6,3],
        [5.599999904632568,6,3],
        [12,6,3],
        [12,6,3],
        [12,-6,3],
        [5.599999904632568,-6,3],
        [-12,6,-3],
        [-12,-6,-3],
        [-3.999999761581421,-6,-3],
        [-3.999999761581421,-6,-3],
        [-3.999999761581421,6,-3],
        [-12,6,-3],
        [3.999999761581421,-6,-3],
        [3.999999761581421,6,-3],
        [-3.999999761581421,6,-3],
        [-3.999999761581421,6,-3],
        [-3.999999761581421,-6,-3],
        [3.999999761581421,-6,-3],
        [5.599999904632568,-6,3],
        [12,-6,3],
        [12,-6,0],
        [12,-6,0],
        [8.600000381469727,-6,0],
        [5.599999904632568,-6,3],
        [-5.599999904632568,6,3],
        [-12,6,3],
        [-12,6,0],
        [-12,6,0],
        [-8.600000381469727,6,0],
        [-5.599999904632568,6,3],
        [3.999999761581421,6,-0.10000008344650269],
        [-3.999999761581421,6,-0.10000008344650269],
        [-3.999999761581421,6,-3],
        [-3.999999761581421,6,-3],
        [3.999999761581421,6,-3],
        [3.999999761581421,6,-0.10000008344650269],
        [-12,6,0],
        [-12,6,-3],
        [-3.999999761581421,6,-3],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-8.600000381469727,6,0],
        [-12,6,0],
        [-3.999999761581421,6,-3],
        [-3.999999761581421,6,-0.10000008344650269],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-3.999999761581421,6,-3],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-12,6,0],
        [-3.999999761581421,-6,-0.10000008344650269],
        [3.999999761581421,-6,-0.10000008344650269],
        [3.999999761581421,-6,-3],
        [3.999999761581421,-6,-3],
        [-3.999999761581421,-6,-3],
        [-3.999999761581421,-6,-0.10000008344650269],
        [12,-6,0],
        [12,-6,-3],
        [3.999999761581421,-6,-3],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [8.600000381469727,-6,0],
        [12,-6,0],
        [3.999999761581421,-6,-3],
        [3.999999761581421,-6,-0.10000008344650269],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [3.999999761581421,-6,-3],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [12,-6,0],
        [-3.999999761581421,-6,-0.10000008344650269],
        [-3.999999761581421,-6,-3],
        [-12,-6,-3],
        [-12,-6,0],
        [-8.600000381469727,-6,0],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-3.999999761581421,-6,-0.10000008344650269],
        [-12,-6,-3],
        [-12,-6,-3],
        [-12,-6,0],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [12,-6,0],
        [12,6,0],
        [12,6,-3],
        [12,6,-3],
        [12,-6,-3],
        [12,-6,0],
        [3.999999761581421,6,-0.10000008344650269],
        [3.999999761581421,6,-3],
        [12,6,-3],
        [12,6,0],
        [8.600000381469727,6,0],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [3.999999761581421,6,-0.10000008344650269],
        [12,6,-3],
        [12,6,-3],
        [12,6,0],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-12,6,0],
        [-12,-6,0],
        [-12,-6,-3],
        [-12,-6,-3],
        [-12,6,-3],
        [-12,6,0],
        [12,6,3],
        [5.599999904632568,6,3],
        [8.600000381469727,6,0],
        [8.600000381469727,6,0],
        [12,6,0],
        [12,6,3],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [8.600000381469727,6,0],
        [5.599999904632568,6,3],
        [5.599999904632568,-6,3],
        [8.600000381469727,-6,0],
        [8.600000381469727,-6,0],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [8.600000381469727,6,0],
        [8.600000381469727,6,0],
        [5.599999904632568,6,3],
        [8.600000381469727,-6,0],
        [-12,-6,3],
        [-5.599999904632568,-6,3],
        [-8.600000381469727,-6,0],
        [-8.600000381469727,-6,0],
        [-12,-6,0],
        [-12,-6,3],
        [-12,6,3],
        [-5.599999904632568,6,3],
        [-5.599999904632568,-6,3],
        [-5.599999904632568,-6,3],
        [-12,-6,3],
        [-12,6,3],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-8.600000381469727,-6,0],
        [-5.599999904632568,-6,3],
        [-5.599999904632568,6,3],
        [-8.600000381469727,6,0],
        [-8.600000381469727,6,0],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-8.600000381469727,-6,0],
        [-8.600000381469727,-6,0],
        [-5.599999904632568,-6,3],
        [-8.600000381469727,6,0],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-8.600000381469727,6.000001430511475,-0.10000008344650269],
        [-3.999999761581421,6,-0.10000008344650269],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-3.999999761581421,-6,-0.10000008344650269],
        [3.999999761581421,-6,-0.10000008344650269],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [3.999999761581421,-6,-0.10000008344650269],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149],
        [-3.999999761581421,6,-0.10000008344650269],
        [3.999999761581421,6,-0.10000008344650269],
        [3.999999761581421,6,-0.10000008344650269],
        [8.600000381469727,6.000001430511475,-0.10000008344650269],
        [3.999999761581421,-6,-0.10000008344650269],
        [3.999999761581421,6,-0.10000008344650269],
        [3.999999761581421,-6,-0.10000008344650269],
        [-8.600000381469727,-5.999999523162842,-0.10000006854534149]],
    polygons: [
        [0,1,2],
        [3,4,5],
        [6,7,8],
        [9,10,11],
        [12,13,14],
        [15,16,17],
        [18,19,20],
        [21,22,23],
        [24,25,26],
        [27,28,29],
        [30,31,32],
        [33,34,35],
        [36,37,38],
        [39,40,41],
        [42,43,44],
        [45,46,47],
        [48,49,50],
        [51,52,53],
        [54,55,56],
        [57,58,59],
        [60,61,62],
        [63,64,65],
        [66,67,68],
        [69,70,71],
        [72,73,74],
        [75,76,77],
        [78,79,80],
        [81,82,83],
        [84,85,86],
        [87,88,89],
        [90,91,92],
        [93,94,95],
        [96,97,98],
        [99,100,101],
        [102,103,104],
        [105,106,107],
        [108,109,110],
        [111,112,113],
        [114,115,116],
        [117,118,119],
        [120,121,122],
        [123,124,125],
        [126,127,128],
        [129,130,131],
        [132,133,134],
        [135,136,137],
        [138,139,140],
        [141,142,143],
        [144,145,146],
        [147,148,149],
        [150,151,152],
        [153,154,155],
        [156,157,158],
        [159,160,161],
        [162,163,164],
        [165,166,167],
        [168,169,170],
        [171,172,173],
        [174,175,176],
        [177,178,179]]
    }));
    return p;
}

function base(radius, notchsize, thickness, smoothing)
{
   // Make the base insert
   const cy = cylinder({h:thickness, r:radius, fn:200});
   const cu = cube({size:[notchsize, notchsize, notchsize+3]});
   return difference(cy, translate([radius-notchsize,-notchsize/2,-1], cu))
}

function CircleInsert(diameter, depth, x, y, z, smoothing)
{
    const cy = cylinder({h:depth, r:diameter/2, fn:smoothing});
    return translate([x,y,z], cy)
}

function RectangleInsert(width, lenght, depth, x, y, z)
{
    const cu = cube({size:[width, lenght, depth]});
    return translate([x,y,z], cu)
}

function DrawWell(b, x, y, z, params){
    // if insert type is a circle
    if (params.insertType == "Circular") {
        b = union(b, CircleInsert(params.embryoRadius, params.embryoDepth, x, y, z, params.smoothing));
    }
    // if insert type is a rectangle
    if (insertType == "Rectangular") {
        b = union(b, RectangleInsert(params.embryoLengthX, params.embryoLengthY, params.embryoDepth, x, y, z)) ;
    }
    return b
}

function main(params)
{
    radius = params.diameter/2; //for a radius of 35mm
    notchsize = params.notchsize;
    thickness = params.thickness;
    smoothing = params.smoothing;
    insertType = params.insertType;
    embryoDepth = params.embryoDepth;
    embryoRadius = params.embryoRadius/2;
    embryoLengthY = params.embryoLengthY;
    embryoLengthX = params.embryoLengthX;

    b = base(radius, notchsize, thickness, smoothing);

    z = thickness;
    row = params.row;
    column = params.column;
    spacing_x = params.spacing_x;
    spacing_y = params.spacing_y;
    y_origin = (- (column) * (2*embryoRadius + spacing_y))/2;
    x_origin = (- (row) * (2*embryoRadius + spacing_x))/2;
    skip = params.gridtype;
    alleys_bool = params.alleys_bool;
    skiprow = params.skiprow;
    skipcol = params.skipcol;

    if (abs(x_origin) < radius * sin(45)) {
        for (i=0; i<row; i+=1) {
            for (j=0; j<column; j+=1) {
                y = y_origin + j * (2*embryoRadius + spacing_y);
                x = x_origin + i * (2*embryoRadius + spacing_x);
                if (alleys_bool == "alleys_yes") {
                    if (abs(i % skiprow) !== 0 && abs(j % skipcol) !== 0) {
                        if (skip == "corner_omitted"){
                            if ((i+1) % skiprow !== 0 || (j+1) % skipcol !== 0) {
                                b = DrawWell(b, x, y, z, params);
                            }
                        }
                        else{
                            b = DrawWell(b, x, y, z, params);
                        }

                    }
                }
                else if (alleys_bool == "alleys_no") {
                    if (skip == "corner_omitted"){
                        if (i != row-1 || j != column-1) {
                           b = DrawWell(b, x, y, z, params);
                        }
                    }
                    else{
                        b = DrawWell(b, x, y, z, params);
                    }
                }
            }
        }
    }
    else {
        echo("Grid size parameter out of range");
    }
    p = getadapter().translate([0,0,3]).rotateX(180).rotateZ(90);
    return [b, p]
}
